#include "spi.h"
#include "errno.h"

/**
  * @brief Функция отправки заданного пользователем количества данных по SPI. 
  * (Пользователь самостоятельно конфигурирует порты ввода-вывода).
  * @param  SPI : Выбор необходимого номера SPI. 
  *   Данный параметр может принимать одно из следующих значений:
  *   SPI0, SPI1.
  * @param  data: указатель на данные для передачи(данные должны быть выравнены
  *  в зависимости от настроек длины слова в регистре управления: 
  * 4 - 8 битные значения считываются как unsigned char,
  * 9 - 16 битные как unsigned short, 32 битные значения как unsigned long).
  * @param  size: размер данных для передачи.
  * @param  timeout: установка максимально разрешенного времени для отправки данных в случае работы в режиме - ведомый,
  * если параметр равен 0, то контроль времени отправки не производится.  
  * @retval код ошибки
  */
int mc_spi_send(SPI_TypeDef *SPI, void *data, unsigned int size, unsigned int timeout)
{
    unsigned int control, shift, spi_error, time;
    unsigned char word_length, mode;
    unsigned char *p_char;
    unsigned short *p_short;
    unsigned int *p_int;
	//Вычитываем конфигурацию SPI
    control = SPI->CR;
    //определяем режим работы (ведущий или ведомый)
    if(control & (1<<25) != 0) mode = 1; //ведущий
    else mode = 0; //ведомый
    
        //определяем длину слова
        if(((control & 0xF00000)>>20) < 3)
        {
            switch((control & 0xF00000)>>20)
            {
                case: 0
                    word_length = 32;
                    break;
                case: 1
                    return ESPI_CR;
                    break;
                case: 2
                    return ESPI_CR;
                    break;
            }
        }
        else
        {
            word_length =  ((control & 0xF00000)>>20) + 1;
        }
        //определяем формат REV и общий сдвиг
        if(SPI->CR & (1<<26) != 0) shift = 32 - word_length;
        else shift = 0;
        //очищаем бит LT на всякий случай, уведомляющий о передаче последнего слова
        SPI->ST = SPI->ST | (1<<14);
            if(mode == 0) 
            {
                mc_start_tick(); //в ведомом режиме запускаем таймер
                time = mc_get_ms_tick();
            }
        //вычитываем в зависимости от длины слова данные из массива
        if(word_length < 9) 
        {
            //считываем как unsigned char
            p_char = (unsigned char*)data;
            while(size != 0)
            {
                //проверяем, что есть свободное место
                while(SPI_ST_NF(SPI) == 0);
                //записываем значение
                SPI->TX = ((*p_char)++)<<shift;
                size--;
                if(mode == 0)
                {
                    if((mc_get_ms_tick() - time) > timeout ) return ESPI_TIMEOUT; //время на передачу вышло
                }
            }
        }
        if(word_length > 8 && word_length < 17) 
        {
            //считываем как unsigned short
            p_short = (unsigned short*)data;
            while(size != 0)
            {
                //проверяем, что есть свободное место
                while(SPI_ST_NF(SPI) == 0);
                //записываем значение
                SPI->TX = ((*p_short)++)<<shift;
                size--;
                if(mode == 0)
                {
                    if((mc_get_ms_tick() - time) > timeout ) return ESPI_TIMEOUT; //время на передачу вышло
                }
            }
        }
        if(word_length > 16 && word_length < 33) 
        {
            //считываем как unsigned int
            p_int = (unsigned short*)data;
            while(size != 0)
            {
                //проверяем, что есть свободное место
                while(SPI_ST_NF(SPI) == 0);
                //записываем значение
                SPI->TX = ((*p_int)++)<<shift;
                size--;
                if(mode == 0)
                {
                    if((mc_get_ms_tick() - time) > timeout ) return ESPI_TIMEOUT; //время на передачу вышло
                }
            }
        }
        //ожидаем передачи последнего слова
        while(SPI_ST_LT(SPI) == 0 && SPI_ST_TIP(SPI) == 1)
        {
            if(mode == 0)
             {
                 if((mc_get_ms_tick() - time) > timeout ) return ESPI_TIMEOUT; //время на передачу вышло
             }
        }
        //очищаем бит LT, уведомляющий о передаче последнего слова
        SPI->ST = SPI->ST | (1<<14);
        return ENOERR; //успешно
	
}

/**
  * @brief Функция отправки заданного пользователем количества данных по SPI и приема ответных данных. 
  * (Пользователь самостоятельно конфигурирует порты ввода-вывода).
  * @param  SPI : Выбор необходимого номера SPI. 
  *   Данный параметр может принимать одно из следующих значений:
  *   SPI0, SPI1.
  * @param  txdata: указатель на данные для передачи(данные должны быть выравнены
  *  в зависимости от настроек длины слова в регистре управления: 
  * 4 - 8 битные значения считываются как unsigned char,
  * 9 - 16 битные как unsigned short, 32 битные значения как unsigned long).
  * @param  rxdata: указатель на принятые данные(данные должны быть выравнены
  *  в зависимости от настроек длины слова в регистре управления: 
  * 4 - 8 битные значения считываются как unsigned char,
  * 9 - 16 битные как unsigned short, 32 битные значения как unsigned long).
  * @param  size: размер данных для передачи.
  * @param  timeout: установка максимально разрешенного времени для отправки данных в случае работы в режиме - ведомый,
  * если параметр равен 0, то контроль времени отправки не производится.  
  * @retval код ошибки
  */
int mc_spi_sendrecv(SPI_TypeDef *SPI, void *txdata, void *rxdata, unsigned int size, unsigned int timeout)
{
    unsigned int control, shift_tx, shift_rx, spi_error, time;
    unsigned char word_length, mode;
    unsigned char *p_char_tx, *p_char_rx;
    unsigned short *p_short_tx, *p_short_rx;
    unsigned int *p_int_tx, *p_int_rx;
	//Вычитываем конфигурацию SPI
    control = SPI->CR;
    //определяем режим работы (ведущий или ведомый)
    if(control & (1<<25) != 0) mode = 1; //ведущий
    else mode = 0; //ведомый
    
        //определяем длину слова
        if(((control & 0xF00000)>>20) < 3)
        {
            switch((control & 0xF00000)>>20)
            {
                case: 0
                    word_length = 32;
                    break;
                case: 1
                    return ESPI_CR;
                    break;
                case: 2
                    return ESPI_CR;
                    break;
            }
        }
        else
        {
            word_length =  ((control & 0xF00000)>>20) + 1;
        }
        //определяем формат REV и общий сдвиг
        if(SPI->CR & (1<<26) != 0) 
        {
            shift_tx = 32 - word_length;
            if(word_length == 32 ) shift_rx = 0;
            else shift_rx = 16;
        }
        else 
        {
            shift_tx = 0;
            shift_rx = 16 - word_length;;
        }
        //очищаем бит LT на всякий случай, уведомляющий о передаче последнего слова
        SPI->ST = SPI->ST | (1<<14);
            if(mode == 0) 
            {
                mc_start_tick(); //в ведомом режиме запускаем таймер
                time = mc_get_ms_tick();
            }
        //вычитываем в зависимости от длины слова данные из массива
        if(word_length < 9) 
        {
            //считываем как unsigned char
            p_char_tx = (unsigned char*)txdata;
            p_char_rx = (unsigned char*)rxdata;
            while(size != 0)
            {
                //проверяем, что есть свободное место
                while(SPI_ST_NF(SPI) == 0);
                //записываем значение
                SPI->TX = ((*p_char_tx)++)<<shift_tx;
                size--;
                //принимаем данные в процессе отправки
                if(SPI_ST_NE(SPI) == 1)
                {
                    *p_char_rx = (SPI->RX)>>shift_rx;
                }
                if(mode == 0)
                {
                    if((mc_get_ms_tick() - time) > timeout ) return ESPI_TIMEOUT; //время на передачу вышло
                }
            }
            //осуществляем прием оставшихся данных
            while(SPI_ST_NE(SPI) == 1)
            {
                *p_char_rx = SPI->RX
            }
        }
        if(word_length > 8 && word_length < 17) 
        {
            //считываем как unsigned short
            p_short_tx = (unsigned short*)txdata;
            p_short_rx = (unsigned short*)rxdata;
            while(size != 0)
            {
                //проверяем, что есть свободное место
                while(SPI_ST_NF(SPI) == 0);
                //записываем значение
                SPI->TX = ((*p_short)++)<<shift_tx;
                size--;
                 //принимаем данные в процессе отправки
                if(SPI_ST_NE(SPI) == 1)
                {
                    *p_short_rx = (SPI->RX)>>shift_rx;
                }
                if(mode == 0)
                {
                    if((mc_get_ms_tick() - time) > timeout ) return ESPI_TIMEOUT; //время на передачу вышло
                }
            }
            //осуществляем прием оставшихся данных
            while(SPI_ST_NE(SPI) == 1)
            {
                *p_short_rx = (SPI->RX)>>shift_rx;
            }
        }
        if(word_length > 16 && word_length < 33) 
        {
            //считываем как unsigned int
            p_int_tx = (unsigned short*)txdata;
            p_int_rx = (unsigned short*)rxdata;
            while(size != 0)
            {
                //проверяем, что есть свободное место
                while(SPI_ST_NF(SPI) == 0);
                //записываем значение
                SPI->TX = ((*p_int)++)<<shift_tx;
                size--;
                if(SPI_ST_NE(SPI) == 1)
                {
                    *p_int_rx = (SPI->RX)>>shift_rx;
                }
                if(mode == 0)
                {
                    if((mc_get_ms_tick() - time) > timeout ) return ESPI_TIMEOUT; //время на передачу вышло
                }
            }
            //осуществляем прием оставшихся данных
            while(SPI_ST_NE(SPI) == 1)
            {
                *p_int_rx = (SPI->RX)>>shift_rx;
            }
        }
        //ожидаем передачи последнего слова
        while(SPI_ST_LT(SPI) == 0 || SPI_ST_TIP(SPI) == 1)
        {
            if(mode == 0)
             {
                 if((mc_get_ms_tick() - time) > timeout ) return ESPI_TIMEOUT; //время на передачу вышло
             }
        }
        //очищаем бит LT, уведомляющий о передаче последнего слова
        SPI->ST = SPI->ST | (1<<14);
        return ENOERR; //успешно
	
}

/**
  * @brief Функция отправки байта данных по SPI. 
  * (Пользователь самостоятельно конфигурирует порты ввода-вывода).
  * @param  SPI : Выбор необходимого номера SPI. 
  *   Данный параметр может принимать одно из следующих значений:
  *   SPI0, SPI1.
  * @param  data: байт данных для передачи.
  * @param  size: размер данных для передачи.
  * @param  timeout: установка максимально разрешенного времени для отправки данных в случае работы в режиме - ведомый,
  * если параметр равен 0, то контроль времени отправки не производится.  
  * @retval Нет
  */
void mc_spi_send_byte(SPI_TypeDef *SPI, unsigned char data)
{
    
}

/**
  * @brief Функция отправки байта данных по SPI. 
  * (Пользователь самостоятельно конфигурирует порты ввода-вывода).
  * @param  SPI : Выбор необходимого номера SPI. 
  *   Данный параметр может принимать одно из следующих значений:
  *   SPI0, SPI1.
  * @param  data: байт данных для передачи.
  * @param  size: размер данных для передачи.
  * @param  timeout: установка максимально разрешенного времени для отправки данных в случае работы в режиме - ведомый,
  * если параметр равен 0, то контроль времени отправки не производится.  
  * @retval Нет
  */
void mc_spi_sendrecv_byte(SPI_TypeDef *SPI, unsigned char data)
{

}



